#
#	Re-encode the PS version of a PFB Type 1 font for a .cod encoding
#
#	Copyright 1996 Richard J. Kinch
#	Kinch, August 1995
#
#	Use:	echo font.ps encoding.cod | awk -f reencode.awk
#
#
#	Note: inserts "space" at code 32, and a charstring for it, and
#	a charstring for .notdef, if not used.
#
BEGIN {
#
#   Hex to decimal conversion table
#
    val["0"] = 0
    val["1"] = 1
    val["2"] = 2
    val["3"] = 3
    val["4"] = 4
    val["5"] = 5
    val["6"] = 6
    val["7"] = 7
    val["8"] = 8
    val["9"] = 9
    val["a"] = 10
    val["A"] = 10
    val["b"] = 11
    val["B"] = 11
    val["c"] = 12
    val["C"] = 12
    val["d"] = 13
    val["D"] = 13
    val["e"] = 14
    val["E"] = 14
    val["f"] = 15
    val["F"] = 15
#
#   Aliases of various PS names to TrueTeX standard
#
    alias["Lquoteright"] = "Lcaron"
    alias["Eng"] = "Ng"
    alias["Germandbls"] = "SS"
    alias["compoundwordmark"] = "compwordmark"
    alias["dquoteright"] = "dcaron"
    alias["lquoteright"] = "lcaron"
    alias["eng"] = "ng"
    alias["perthousand"] = "perthousandzero"
    alias["tquoteright"] = "tcaron"
    alias["hyphenchar"] = "hyphen"
    }
{
	font_name = $1	# Font name
	ps_file = $2	# Input PFB file (already in PS form via t1utils)
	cod_file = $3	# TrueTeX encoding file
#
#	Read .cod encoding table
#
	while ((getline < cod_file) > 0) {
	    code = val[substr($1,1,1)]*16 + val[substr($1,2,1)]
	    psname = $2
	    nameof[code] = psname
	    code_plus_one_of[psname] = code+1
	    }
	close(cod_file)
#
#	Fix up "space" encoding if missing
#
	if (!code_plus_one_of["space"]) {
	    nameof[32] = "space"
	    code_plus_one_of["space"] = 32+1
	    }
#
#	1st pass of ps file to get Charstrings list
#
	ps_names = 0
	while ((getline < ps_file) > 0) {
	    if ($0~/^\/[\.A-Za-z0-9][\.A-Za-z0-9]* {$/) {	# }
		this_name = substr($1,2)
		if (alias[this_name]!="") this_name = alias[this_name]
#
#		ps_name_index[glyphname] is an index > 0 for each glyph
#		charstring in the font.  The index is initially a sequence
#		number from 1 to the number of charstrings.  Later when
#		we output the encoding we'll negate it to indicate that it
#		was used in the encoding.
#
		ps_name_index[this_name] = ps_names+1
		ps_names++
		}
	    }
	close(ps_file)
#
#	2nd pass of ps file to emit reencoded version
#
	while ((getline < ps_file) > 0) {
#
#	    Test for end of skipped charstring
#
	    if (skipping) {
		if (substr($0,length($0)-1)=="|-") skipping = 0
		continue
		}
#
#	    Get rid of troublesome items
#
	    if ($0~/^%!/) {
		printf "%%!PS-AdobeFont-1.1: %s 1.0\n", font_name
		continue
		}
	    else if ($0~/^%/) continue
	    else if ($0~/version/) {
		printf "/version (1.0) readonly def\n"
		continue
		}
	    else if ($0~/Notice/) {
		printf "/Notice (Copyright 1995, Richard Kinch) readonly def\n"
		continue
		}
	    else if ($0~/FullName/) {
		printf "/FullName (%s) readonly def\n", font_name
		continue
		}
	    else if ($0~/FamilyName/) {
		printf "/FamilyName (%s) readonly def\n", font_name
		continue
		}
	    else if ($0~/FontName.*def$/) {
		printf "/FontName /%s def\n", font_name
		continue
		}
	    else if ($0~/Weight/) {
		continue
		}
#
#	    Test if found an Encoding entry
#
	    else if ($0~/^dup *[0-9][0-9]* *\/[a-zA-Z0-9][a-zA-Z0-9]* *put$/) {
		if (shown_encoding) continue
		else {
		    for (i in nameof) {
			this_name = nameof[i]
			if (alias[this_name]!="") this_name = alias[this_name]
			if (!ps_name_index[this_name] && this_name!="space") \
				printf \
				"No charstring in PFB for \"%s\"\n", \
				this_name > "/dev/stderr"
			else {
			    printf "dup %d /%s put\n", i, this_name
			    if (ps_name_index[this_name]>0) {
				ps_name_index[this_name] = \
					-ps_name_index[this_name]
				}
			    }
			}
		    shown_encoding = 1
		    }
		}
#
#	    Found start of a charstring.  Alias name if needed, or begin
#	    omitting unencoded character.
#
	    else if ($0~/^\/[\.A-Za-z0-9][\.A-Za-z0-9]* {$/) { # }
		this_name = substr($1,2)
		if (alias[this_name]!="") {
		    this_name = alias[this_name]
		    }
#
#		Test if start of an unencoded charstring to be skipped
#
		if (!code_plus_one_of[this_name]) skipping = 1
		else printf "/%s {\n", this_name	# }
		}
#
#	    End of charstrings.  Do any fix-ups
#
	    else if ($0=="end") {
		if (ps_name_index["space"]>=0) {
		    printf "/space {\n" \
			"	0 1000 3 div hsbw\n" \
			"	endchar\n" \
			"	} |-\n"
		    ps_name_index["space"] = -33
		    }
		print
		}
	    else print
	    }
	close(ps_file)
	exit
}
#END {
#    for (i in ps_name_index) {
#	if (ps_name_index[i]>0) printf "Unencoded in new font: %s\n", \
#		i > "/dev/stderr"
#	}
#    }
